#!/bin/bash
# myDevices setup script
set -x
exec 2>/var/log/myDevicesSetup
SEARCH="python3"
FOUND=""
INSTALLED=""
CONFIG_FILE=/etc/myDevices/AppSettings.ini
PLATFORM="raspberrypi"
args=("$@")
reboot_args="$@"
ELEMENTS=${#args[@]}
declare -a ASK_TO_REBOOT
ASK_TO_REBOOT=0
VERSION="2.0.1.255"
TARGET_KEY=Version
UPDATE=0
EXPAND_FILE_SYSTEM=0
INVITECODE=""
NETWORK_SETTINGS="live_network.ini"
ENV="live"
nonverbose='-ne'
nonverboseend='\r'
installpath=/etc/myDevices
uninstallpath=${installpath}/uninstall
#TODO update progress server for cayenne is ready
#also all servers should be updated
SERVER='https://cayenne.mydevices.com'
UPDATE_OBJECT='/cayenne/api/install'
#DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
DIR="${0%/*}"
cd "${DIR}"
declare echoLog='silentEcho'
export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
function silentEcho() {
    :
}
function normalEcho(){
    echo "$(date) $@"
}
############################################################################################################
#Make sure all params are gathered correcly before doing any progress report or other stuff
for (( i=0;i<$ELEMENTS;i++)); do
    if [ "${args[${i}]}" == "-code" ]; then
    ((i++))
    INVITECODE="${args[${i}]}"
    fi
    if [ "${args[${i}]}" == "-update" ]; then
    UPDATE=1
    fi

    if [ "${args[${i}]}" == "-expandfs" ]; then
    EXPAND_FILE_SYSTEM=1
    fi    
    
    if [ "${args[${i}]}" == "-dev" ]; then
    NETWORK_SETTINGS="dev_network.ini"
    ENV="dev"
    SERVER='https://cayenne-dev.mydevices.com'
    fi

    if [ "${args[${i}]}" == "-stg" ]; then
    NETWORK_SETTINGS="stg_network.ini"
    ENV="stg"
    SERVER='https://cayenne-staging.mydevices.com'
    fi

    if [ "${args[${i}]}" == "-qa2" ]; then
    NETWORK_SETTINGS="qa2_network.ini"
    ENV="qa2"
    SERVER='https://cayenne-qa2.mydevices.com'
    fi

    if [ "${args[${i}]}" == "-qa1" ]; then
    NETWORK_SETTINGS="qa1_network.ini"
    ENV="qa1"
    SERVER='https://cayenne-qa1.mydevices.com'
    fi

    if [ "${args[${i}]}" == "-platform" ]; then
    ((i++))
    PLATFORM="${args[${i}]}"
    fi
done;
#DONE params parsing
############################################################################################################

function updateProgress(){
    echo $nonverbose "($1%)$nonverboseend"
    if [ -n "$3" ]; then
        $echoLog "$3"
        if [ -n "$INVITECODE" ] && [ $UPDATE == 0 ]; then
            sudo nohup curl --data "invitecode=$INVITECODE&progress=$1&step=$2" $SERVER$UPDATE_OBJECT &> /dev/null
        fi
    fi
}

while [[ $# > 0 ]]; do
    case "$1" in
        -v|--verbose) echoLog='normalEcho'; nonverbose=''; nonverboseend=''; ;;
    esac
    shift;
done

updateProgress 0 0 'Installing myDevices agent'

function expandFileSystem(){
    if [ $EXPAND_FILE_SYSTEM == 0 ]; then
        return 0
    fi
    $echoLog "Expanding file system"
    #If the partition never got expanded - continue with expanding the partition 
    #this should fix NOOBS problem with already expanded partitions locking the installer
    isPythonInstalled=`which parted`
    if [ -z "$isPythonInstalled" ]; then
        return 1
    fi
    local retVal=0
    freespace=`parted /dev/mmcblk0 unit % print free | grep 'Free Space' | awk '{print $2}'`
    for i in $freespace
    do
        #this means there is a partitions of the end of last partition    
        if [ $i = "100%" ]; then
            sudo raspi-config --expand-rootfs &>1
            retVal=1
        fi
    done
    echo $retVal
    return 0
}

reboot_script="${DIR}/reboot_install.sh"
reboot_installer="${reboot_script} ${reboot_args}"

#make sure we have enough space before or after restart
availableSize=`df -k | head -2 | tail -1 | awk '{print $4}'`
if [ $availableSize -lt 200000 ]; then
    retVal=$(expandFileSystem)
    echo $retVal
    if [ $retVal = 1 ]; then
        updateProgress -2 -2 'Setup will continue after reboot!'
        #setup crontab to start at reboot time
        chmod 0755 $reboot_script          
        line="@reboot ${reboot_installer}"
        (sudo crontab -u root -l; echo "$line" ) | sudo crontab -u root -
        sleep 3
        sudo reboot
        exit 2
    fi
    updateProgress -1 -1 'Error installing agent'
    echo "Setup requires at least 200Mb for install" && exit 1
else
    expandFileSystem
fi

# #now delete crontab if was setup
if [ -f $reboot_script ]; then
    out=`crontab -l | grep -v "@reboot ${reboot_script}" | crontab -`
fi

mkdir /etc/myDevices/
mkdir -p "${uninstallpath}"

if [ "${INVITECODE}" == "" ] && [ $UPDATE == 0 ]; then
    $echoLog "ERROR: invite code is missing your setup file and myDevices cannot continue without invite code: setup.sh -code <code> "
    exit 1
else
if [ $UPDATE == 0 ]; then
    $echoLog 'Installing system libraries'
    declare -a dependencies=("python3" "python3-dev" "python3-setuptools")
    currentProgress=0
    dependency_count=${#dependencies[@]}
    if [ $dependency_count -eq 0 ]; then dependency_count=1; fi
    #next step is at 50% -> so split the 50 in equal parts
    packagestep=$((50 / dependency_count))
    for dependency in "${dependencies[@]}"
    do
        PACKAGE_INSTALLED=$(dpkg-query -W --showformat='${Status}\n' $dependency | grep "install ok installed")
        if [ "" == "$PACKAGE_INSTALLED" ]; then
            sudo apt-get --yes install $dependency
            touch "${uninstallpath}/installed_package"
            echo "$dependency" >> "${uninstallpath}/installed_package"
            currentProgress=$((currentProgress+packagestep))
            updateProgress $currentProgress 1 "Installed package: $dependency"
        else
            currentProgress=$((currentProgress+packagestep))
            updateProgress $currentProgress 1 "Found package: $dependency"
        fi
    done

if [ -n "$INVITECODE" ] && grep -q -i "InviteCode.*=.*$INVITECODE" $CONFIG_FILE; then
    #Invite code already exists so we don't overwrite the config file
    sed  -i "s/\($TARGET_KEY *= *\).*/\1$VERSION/" $CONFIG_FILE
else
touch $CONFIG_FILE
cat > $CONFIG_FILE << POST_CONF 
[Agent]
InviteCode=${INVITECODE}
Initialized=false
Version=${VERSION}
Environment=${ENV}
UpdateProgress=${SERVER}${UPDATE_OBJECT}
POST_CONF
fi

else
    sed  -i "s/\($TARGET_KEY *= *\).*/\1$VERSION/" $CONFIG_FILE
fi
fi

# Look up for installed python
python=''
for py in $SEARCH; do
    program="/usr/bin/$py"
    if [ -x $program ]; then
        FOUND="$FOUND $py"
        version=`$py -V 2>&1`
        include=`$py -c "import distutils.sysconfig; print(distutils.sysconfig.get_python_inc())"`
        $echoLog "Found $version"
        python=$py
    fi
done


updateProgress 50 2 "Installing agent files"
if pgrep "systemd" > /dev/null; then
    # Add systemd service
    systemctl disable myDevices
    rm -rf /etc/systemd/system/myDevices.service
    $echoLog "Installing systemd service"
    cp -rf myDevices.service /etc/systemd/system/
    sed -i "s/python/$python/g" /etc/systemd/system/myDevices.service       
    sed -i "s|\([=-]\)mkdir|\1$(which mkdir)|g" /etc/systemd/system/myDevices.service
    sed -i "s|\([=-]\)chown|\1$(which chown)|g" /etc/systemd/system/myDevices.service
    # for dev env enable debug
    if [ "$ENV" == "dev" ]; then
        sed -i "s/-m myDevices/-m myDevices -d/g" /etc/systemd/system/myDevices.service
    fi
    rm -rf /etc/init.d/myDevices
    systemctl enable myDevices
    systemctl daemon-reload
else
    # Add init.d service/daemon script
    rm -rf /etc/init.d/myDevices
    $echoLog "Installing init.d startup script"
    cp -rf myDevices.init.sh /etc/init.d/myDevices
    sed -i "s/python/$python/g" /etc/init.d/myDevices
    #for environment other than live add debug
    if [ "$ENV" == "dev" ]; then
        sed -i "s/-m myDevices/-m myDevices -d/g" /etc/init.d/myDevices
    fi
    chmod 0755 /etc/init.d/myDevices
fi

mkdir -p /etc/myDevices/scripts
cp -rf tightvncsetup.sh /etc/myDevices/scripts

# Add myDevices command
rm -rf /usr/bin/myDevices
cp -rf myDevices.sh /usr/bin/myDevices
sed -i "s/python/$python/g" /usr/bin/myDevices
chmod 0755 /usr/bin/myDevices

cp -rf $NETWORK_SETTINGS /etc/myDevices/Network.ini
chmod 0755 /etc/myDevices/Network.ini

cp -rf ca.crt /etc/myDevices/ca.crt
chmod 0755 /etc/myDevices/ca.crt

cp -rf uninstall.sh "${uninstallpath}/uninstall.sh"
chmod 0755 "${uninstallpath}/uninstall.sh"

cp -rf crontab.sh /etc/myDevices/crontab.sh
chmod 0755 /etc/myDevices/crontab.sh    



updateProgress 60 3 'Installing software components'

# Create cayenne user
adduser -q --system --disabled-password -shell $SHELL --group cayenne
usermod -a -G gpio,i2c,spi cayenne

cd Cayenne-Agent
# Try to compile and install for the current python
if [ -f "$include/Python.h" ]; then
    $echoLog "Trying to install myDevices for $python version: $version"

    $python setup.py install --force --record "${uninstallpath}/installed_agent"
    if [ "$?" -ne "0" ]; then
        #setup error, continue with next python
        $echoLog "Build for $version failed"
        exit 1
    fi

    $echoLog "myDevices installed for $version"
    INSTALLED="$INSTALLED $python"
else
    $echoLog "Cannot install for $version : missing development headers"
fi
# Go back to the install folder
cd ..

# Ensure myDevices is installed to continue
if [ -z "$INSTALLED" ]; then
    if [ -z "$FOUND" ]; then
        $echoLog "ERROR: myDevices cannot be installed - python3 cannot found"
        exit 1
    else
        $echoLog "ERROR: myDevices cannot be installed - please check errors above"
        exit 2
    fi
fi

updateProgress 98 4 'Configuring os and agent'

# Set up sudoers rules
SUDOERS_CONFIG=cayenne-sudoers
sed -i "s/python /$python /g" $SUDOERS_CONFIG
visudo -c -q -f $SUDOERS_CONFIG
if [ $? -eq 0 ]
then
    chown root:root $SUDOERS_CONFIG
    chmod 440 $SUDOERS_CONFIG
    cp $SUDOERS_CONFIG /etc/sudoers.d/
else
    $echoLog "$SUDOERS_CONFIG file is not valid, installation cannot continue" >&2
    exit 1
fi

# Set up folders and user permissions
mkdir -p /var/run/myDevices
chown --quiet cayenne:cayenne /var/run/myDevices
mkdir -p /var/log/myDevices
chown -R --quiet cayenne:cayenne /var/log/myDevices
chown --quiet cayenne:cayenne /etc/myDevices
chown --quiet cayenne:cayenne /etc/myDevices/AppSettings.ini
chown --quiet cayenne:cayenne /etc/myDevices/agent.db*
chown --quiet cayenne:cayenne /etc/myDevices/devices.json

#call TightVNC setup
bash tightvncsetup.sh

if [ "$PLATFORM" == "raspberrypi" ]; then     ## GOOD
    if [ $UPDATE == 0 ]; then

        CONFIG=/boot/config.txt
        BLACKLIST=/etc/modprobe.d/raspi-blacklist.conf

        do_w1(){
            output=$( cat $CONFIG | grep w1-gpio )
            if [ -z "$output" ]; then
                echo "dtoverlay=w1-gpio" >> $CONFIG
            else
                sed  -i "s/\(#dtoverlay.*=.*w1-gpio\).*/\dtoverlay=w1-gpio/" $CONFIG
            fi
            ASK_TO_REBOOT=1
        }

        do_spi() {
          DEVICE_TREE="yes" # assume not disabled
          DEFAULT=
          if [ -e $CONFIG ] && grep -q "^device_tree=$" $CONFIG; then
            DEVICE_TREE="no"
          fi
          CURRENT_SETTING="off" # assume disabled
          DEFAULT=--defaultno
          if [ -e $CONFIG ] && grep -q -E "^(device_tree_param|dtparam)=([^,]*,)*spi(=(on|true|yes|1))?(,.*)?$" $CONFIG; then
            CURRENT_SETTING="on"
            DEFAULT=
          fi
          if [ $DEVICE_TREE = "yes" ]; then
            SETTING=on
            STATUS=enabled
            TENSE=is
            REBOOT=
            if [ $SETTING != $CURRENT_SETTING ]; then
              ASK_TO_REBOOT=1
            fi
            sed $CONFIG -i -r -e "s/^((device_tree_param|dtparam)=([^,]*,)*spi)(=[^,]*)?/\1=$SETTING/"
            if ! grep -q -E "^(device_tree_param|dtparam)=([^,]*,)*spi=[^,]*" $CONFIG; then
              printf "dtparam=spi=$SETTING\n" >> $CONFIG
            fi
          fi
          CURRENT_STATUS="yes" # assume not blacklisted
          DEFAULT=
          if [ -e $BLACKLIST ] && grep -q "^blacklist[[:space:]]*spi[-_]bcm2708" $BLACKLIST; then
            CURRENT_STATUS="no"
            DEFAULT=--defaultno
          fi
          if ! [ -e $BLACKLIST ]; then
            touch $BLACKLIST
            sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*spi[-_]bcm2708\)/#\1/"
          fi
        }

        do_i2c() {
          DEVICE_TREE="yes" # assume not disabled
          DEFAULT=
          if [ -e $CONFIG ] && grep -q "^device_tree=$" $CONFIG; then
            DEVICE_TREE="no"
          fi
          CURRENT_SETTING="off" # assume disabled
          DEFAULT=--defaultno
          if [ -e $CONFIG ] && grep -q -E "^(device_tree_param|dtparam)=([^,]*,)*i2c(_arm)?(=(on|true|yes|1))?(,.*)?$" $CONFIG; then
            CURRENT_SETTING="on"
            DEFAULT=
          fi
          if [ $DEVICE_TREE = "yes" ]; then
            SETTING=on
            STATUS=enabled
            if [ $SETTING != $CURRENT_SETTING ]; then
              ASK_TO_REBOOT=1
            fi
            sed $CONFIG -i -r -e "s/^((device_tree_param|dtparam)=([^,]*,)*i2c(_arm)?)(=[^,]*)?/\1=$SETTING/"
            if ! grep -q -E "^(device_tree_param|dtparam)=([^,]*,)*i2c(_arm)?=[^,]*" $CONFIG; then
              printf "dtparam=i2c_arm=$SETTING\n" >> $CONFIG
            fi
          fi
          CURRENT_STATUS="yes" # assume not blacklisted
          DEFAULT=
          if [ -e $BLACKLIST ] && grep -q "^blacklist[[:space:]]*i2c[-_]bcm2708" $BLACKLIST; then
            CURRENT_STATUS="no"
            DEFAULT=--defaultno
          fi
          if ! [ -e $BLACKLIST ]; then
            touch $BLACKLIST
          fi
          sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*i2c[-_]bcm2708\)/#\1/"
          sed $BLACKLIST -i -e "s/^#\(blacklist[[:space:]]*i2c-dev\)/\1/"
          modprobe i2c-bcm2708
          modprobe i2c-dev
        }
        do_spi
        do_i2c
        do_w1

        ###########################################################################################
        ## CRON TAB ADD
        line="* * * * * /etc/myDevices/crontab.sh"
        sudo sed -i "s/.*myDevices.*/ /g" /var/spool/cron/crontabs/root
        (sudo crontab -u root -l; echo "$line" ) | sudo crontab -u root -
        ###########################################################################################
        #make sure cron starts
        sudo /etc/init.d/cron start > /dev/null

        sudo service myDevices start
    else
        $echoLog "Migrate devices"
        OLD_DEVICES_JSON_FILE=/etc/webiopi/devices.json
        NEW_DEVICES_JSON_FILE=/etc/myDevices/devices.json
        if [ ! -f $NEW_DEVICES_JSON_FILE ]; then
            if [ -f $OLD_DEVICES_JSON_FILE ]; then
                #copy the webiopi devices file if we are updating and the agent does not have its own
                cp $OLD_DEVICES_JSON_FILE $NEW_DEVICES_JSON_FILE
            else
                #just create an empty devices file if a webiopi one does not exist
                echo "[]" > $NEW_DEVICES_JSON_FILE
            fi
        fi
        chown --quiet cayenne:cayenne $NEW_DEVICES_JSON_FILE
        WEBIOPI_UNINSTALLED=${uninstallpath}/webiopi_uninstalled
        if [ ! -f $WEBIOPI_UNINSTALLED ]; then
            bash /etc/webiopi/uninstall/uninstall.sh
            systemctl daemon-reload
            date > $WEBIOPI_UNINSTALLED
            #remove old webiopi file since it is not needed
            rm -f /${uninstallpath}/webiopi
        fi
        sudo service myDevices restart
    fi
fi

if [ $UPDATE == 1 ]; then
    rm -rf /etc/myDevices/updates/
fi

# Archive/move/delete old log files
if [ -x /var/log/myDevices.log ]; then
    tar -cvjf /var/log/myDevices/myDevices.log.bz2 -C /var/log myDevices.log
fi
rm -f /var/log/myDevices.log
rm -f /var/log/myDevicesData
rm -f /var/log/myDevices.log.*-*-*.tar.bz2
mv /var/log/myDevices.log* /var/log/myDevices/
mv /var/log/myDevices.update* /var/log/myDevices/
chown --quiet cayenne:cayenne /var/log/myDevices/*

updateProgress 100 5 "Install finished"

rm -rf "${DIR}"
rm -rf "${DIR}".tar.gz


#this is an workround and will make the ios/Android apps to show install finished even when pi reboots
updateProgress 100 6 "Install cleanup finished"

if [ $ASK_TO_REBOOT -eq 1 ] && [ $UPDATE -eq 0 ]; then
    echo 'Your system will reboot!'
    sudo reboot
fi

exit 0
